package com.bonus.system.att.dao;

import com.bonus.system.api.domain.MapVo;
import com.bonus.system.api.domain.SysUser;
import com.bonus.system.att.entity.AttFaceBean;
import com.bonus.system.att.entity.AttSourceDataBean;
import com.bonus.system.att.entity.LeaveBean;
import com.bonus.system.att.entity.OrgChangeBean;
import com.bonus.system.basic.domain.SysOrg;
import com.bonus.system.evection.entity.EvectionBean;
import com.bonus.system.holiday.entity.HolidayBean;
import com.bonus.system.index.entity.MapBean;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 小程序同步定时器
 * @author fly
 */
@Repository("WechatPushDao")
public interface WechatPushDao {

    List<SysUser> getPersonList();

    void updateWebPassword(SysUser wechatUser);

    void updateWebOpenId(SysUser wechatUser);

    List<AttFaceBean> getWebFaceList();

    void insertWebFace(AttFaceBean user);

    void updateWebFace(AttFaceBean user);

    /**
     * 查询请假数据
     * @return
     */
    SysOrg getOrgInfoByUserId(Long userId);

    void insertWebLeave(EvectionBean bean);

    List<EvectionBean> getWebLeaveList();

    void insertWechatLeave(EvectionBean bean);


    SysOrg getOrgInfoByUserName(String userName);

    void pushAttDataToQsy(@Param("list")List<AttSourceDataBean> attSourceList);

    MapBean getProOrgToQsy(AttSourceDataBean c);
}
